
#include "rhythmbox-draw.h"

//Inclusion des variables de dessins
extern Icon *myIcon;
extern CairoDock *myDock;
extern cairo_t *myDrawContext;
extern cairo_surface_t *rhythmbox_pSurface;
extern cairo_surface_t *rhythmbox_pPlaySurface;
extern cairo_surface_t *rhythmbox_pPauseSurface;
extern cairo_surface_t *rhythmbox_pStopSurface;

//Inclusion des variables de configuration
extern gchar *conf_defaultTitle;
extern CairoDockAnimationType conf_changeAnimation;
extern gboolean conf_enableDialogs;
extern gboolean conf_enableCover;
extern double conf_timeDialogs;

//Inclusion des variables d'etats
extern gboolean rhythmbox_opening;
extern gboolean rhythmbox_playing;
extern gboolean cover_exist;
extern gchar *playing_uri;
extern gchar *playing_artist;
extern gchar *playing_album;
extern gchar *playing_title;

void rhythmbox_iconWitness(int animationLenght)
{
	cairo_dock_animate_icon (myIcon, myDock, conf_changeAnimation, animationLenght);
}


//*********************************************************************************
// rhythmbox_changeIconName() : Change l'image de l'icone
//*********************************************************************************
void rhythmbox_setIconSurface(cairo_surface_t *pSurface)
{
	cairo_dock_set_icon_surface_with_reflect (myDrawContext, pSurface, myIcon, myDock);
	
	cairo_dock_redraw_my_icon (myIcon, myDock);
}


//*********************************************************************************
// rhythmbox_changeIconName() : Change le nom de l'icone
//*********************************************************************************
void rhythmbox_setIconName(const gchar *cIconName)
{
	cairo_dock_set_icon_name (myDrawContext, cIconName, myIcon, myDock);
}


void update_icon(gboolean make_witness)
{
	if(rhythmbox_opening)
	{
		gchar *songName,*cover;
		
		songName = g_strdup_printf("%s - %s",playing_artist,playing_title);
		rhythmbox_setIconName(songName);
		g_free (songName);
		
		//Affichage de l'album
		cover = g_strdup_printf("%s/.gnome2/rhythmbox/covers/%s - %s.jpg", g_getenv ("HOME"), playing_artist,playing_album);
		if(g_file_test (cover, G_FILE_TEST_EXISTS) && conf_enableCover)
		{
			cairo_dock_set_image_on_icon (myDrawContext, cover, myIcon, myDock);
			cover_exist = TRUE;
		}
		else
		{
			if(rhythmbox_playing)
			{
				rhythmbox_setIconSurface(rhythmbox_pPlaySurface);
			}
			else
			{
				rhythmbox_setIconSurface(rhythmbox_pPauseSurface);
			}
			cover_exist = FALSE;
		}
		g_free (cover);
		
		if(make_witness)
		{
			rhythmbox_iconWitness(1);
			if(conf_enableDialogs)
			{
				music_dialog();
			}
		}
	}
	else
	{
		rhythmbox_setIconName(conf_defaultTitle);
		rhythmbox_setIconSurface( rhythmbox_pStopSurface );
	}
}

void music_dialog(void)
{
	gchar *message = g_strdup_printf("Artist : %s\nAlbum : %s\nTitle : %s",playing_artist,playing_album,playing_title);
	cairo_dock_show_temporary_dialog (message,myIcon,myDock,conf_timeDialogs);
	g_free (message);
}
