#include <stdlib.h>

#include "cairo-dock-dialogs.h"
#include "rhythmbox-draw.h"
#include "rhythmbox-config.h"
#include "rhythmbox-dbus.h"
#include "rhythmbox-menu-functions.h"
#include "rhythmbox-init.h"

#define MY_APPLET_USER_DATA_DIR "rhythmbox"

Icon *myIcon = NULL;
CairoDock *myDock = NULL;
cairo_t *myDrawContext = NULL;

cairo_surface_t *rhythmbox_pSurface = NULL;
cairo_surface_t *rhythmbox_pPlaySurface = NULL;
cairo_surface_t *rhythmbox_pPauseSurface = NULL;
cairo_surface_t *rhythmbox_pStopSurface = NULL;
cairo_surface_t *rhythmbox_pBrokenSurface = NULL;

gchar *conf_defaultTitle = NULL;
gboolean rhythmbox_dbus_enable = FALSE;
gboolean rhythmbox_opening = FALSE;
gboolean rhythmbox_playing;
gboolean cover_exist = FALSE;
const gchar *playing_uri = NULL;
const gchar *playing_artist = NULL;
const gchar *playing_album = NULL;
const gchar *playing_title = NULL;

CairoDockVisitCard *pre_init (void)
{
	rhythmbox_dbus_enable = rhythmbox_dbus_init();
	
	CairoDockVisitCard *pVisitCard = g_new0 (CairoDockVisitCard, 1);
	pVisitCard->cModuleName = g_strdup ("Rhythmbox");
	pVisitCard->cReadmeFilePath = g_strdup_printf ("%s/%s", MY_APPLET_SHARE_DATA_DIR, MY_APPLET_README_FILE);
	pVisitCard->iMajorVersionNeeded = 1;
	pVisitCard->iMinorVersionNeeded = 4;
	pVisitCard->iMicroVersionNeeded = 5;
	return pVisitCard;
}

Icon *init (CairoDock *pDock, gchar **cConfFilePath, GError **erreur)
{
	g_print ("%s ()\n", __func__);
	myDock = pDock;
	
	//\_______________ On verifie que nos fichiers existent.
	*cConfFilePath = cairo_dock_check_conf_file_exists (MY_APPLET_USER_DATA_DIR, MY_APPLET_SHARE_DATA_DIR, MY_APPLET_CONF_FILE);
	
	//Lecture du fichier de configuration
	int iOriginalWidth = 48, iOriginalHeight = 48;
	gchar *cAppletName = NULL;
	read_conf_file (*cConfFilePath, &iOriginalWidth, &iOriginalHeight, &cAppletName);
	
	//Création de l'icone
	myIcon = cairo_dock_create_icon_for_applet (pDock, iOriginalWidth, iOriginalHeight, cAppletName, NULL);
	myDrawContext = cairo_create (myIcon->pIconBuffer);
	
	conf_defaultTitle = g_strdup (cAppletName);
	
	GString *sImagePath = g_string_new ("");  // ce serait bien de pouvoir choisir ses icones, comme dans l'applet logout...
	//Chargement de l'image "default"
	rhythmbox_pSurface = cairo_dock_load_image_for_icon (myDrawContext,
		g_strdup_printf ("%s/default.svg", MY_APPLET_SHARE_DATA_DIR),
		myIcon->fWidth * (1 + g_fAmplitude),
		myIcon->fHeight * (1 + g_fAmplitude));
	
	//Chargement de l'image "pause"
	rhythmbox_pPauseSurface = cairo_dock_load_image_for_icon (myDrawContext,
		g_strdup_printf ("%s/pause.svg", MY_APPLET_SHARE_DATA_DIR),
		myIcon->fWidth * (1 + g_fAmplitude),
		myIcon->fHeight * (1 + g_fAmplitude));
	
	//Chargement de l'image "play"
	rhythmbox_pPlaySurface = cairo_dock_load_image_for_icon (myDrawContext,
		g_strdup_printf ("%s/play.svg", MY_APPLET_SHARE_DATA_DIR),
		myIcon->fWidth * (1 + g_fAmplitude),
		myIcon->fHeight * (1 + g_fAmplitude));

	//Chargement de l'image "stop"
	rhythmbox_pStopSurface = cairo_dock_load_image_for_icon (myDrawContext,
		g_strdup_printf ("%s/stop.svg", MY_APPLET_SHARE_DATA_DIR),
		myIcon->fWidth * (1 + g_fAmplitude),
		myIcon->fHeight * (1 + g_fAmplitude));
	
	//Chargement de l'image "broken"
	rhythmbox_pBrokenSurface = cairo_dock_load_image_for_icon (myDrawContext,
		g_strdup_printf ("%s/broken.svg", MY_APPLET_SHARE_DATA_DIR),
		myIcon->fWidth * (1 + g_fAmplitude),
		myIcon->fHeight * (1 + g_fAmplitude));
	
	g_string_free (sImagePath, TRUE);
	
	if (rhythmbox_dbus_enable)
		cairo_dock_set_icon_surface_with_reflect (myDrawContext, rhythmbox_pSurface, myIcon, myDock);
	else
		cairo_dock_set_icon_surface_with_reflect (myDrawContext, rhythmbox_pBrokenSurface, myIcon, myDock);

	//dbus_detect_rhythmbox();
	if(rhythmbox_opening)
	{
		playing_uri = rhythmbox_getPlayingUri();
		getSongInfos();
		update_icon( FALSE );
	}
	//Enregistrement des notifications	
	cairo_dock_register_first_notifications (CAIRO_DOCK_CLICK_ICON,
		(CairoDockNotificationFunc) rhythmbox_click,
		CAIRO_DOCK_MIDDLE_CLICK_ICON,
		(CairoDockNotificationFunc) rhythmbox_double_click,
		CAIRO_DOCK_BUILD_MENU,
		(CairoDockNotificationFunc) rhythmbox_notification_build_menu,
		-1);
	
	g_free (cAppletName);
	return myIcon;
}

void stop (void)
{
	g_print ("%s ()\n", __func__);
	
	cairo_dock_remove_notification_func (CAIRO_DOCK_CLICK_ICON, (CairoDockNotificationFunc) rhythmbox_click);
	cairo_dock_remove_notification_func (CAIRO_DOCK_MIDDLE_CLICK_ICON, (CairoDockNotificationFunc) rhythmbox_double_click);
	cairo_dock_remove_notification_func (CAIRO_DOCK_BUILD_MENU, (CairoDockNotificationFunc) rhythmbox_notification_build_menu);
	
	g_free (conf_defaultTitle);
	conf_defaultTitle = NULL;
	
	cairo_surface_destroy (rhythmbox_pSurface);
	rhythmbox_pSurface = NULL;
	cairo_surface_destroy (rhythmbox_pPlaySurface);
	rhythmbox_pPlaySurface = NULL;
	cairo_surface_destroy (rhythmbox_pPauseSurface);
	rhythmbox_pPauseSurface = NULL;
	cairo_surface_destroy (rhythmbox_pStopSurface);
	rhythmbox_pStopSurface = NULL;
	cairo_surface_destroy (rhythmbox_pBrokenSurface);
	rhythmbox_pBrokenSurface = NULL;
	
	playing_uri = NULL;
	playing_artist = NULL;
	playing_album = NULL;
	playing_title = NULL;
	
	myIcon = NULL;
	myDock = NULL;
	cairo_destroy (myDrawContext);
	myDrawContext = NULL;
}
